@@ -22,6 +22,7 @@
 
 #include <qdialog.h>
 #include <qlineedit.h>
+#include <qcombobox.h>
 
 class Database_Create_Dialog : public QDialog {
     Q_OBJECT
@@ -29,7 +30,8 @@ public:
 	Database_Create_Dialog(QWidget *parent, const char *name=0);
 	~Database_Create_Dialog();
 private:
-    QLineEdit *host_line, *db_line, *user_line, *pass_line, *su_line, *supass_line;
+    QLineEdit *host_line, *db_line, *user_line, *pass_line, *su_line, *supass_line, *clienthost_line;
+    QComboBox *comboEncoding;
 
 private slots:
     void slot_create();
