@@ -16,8 +16,15 @@
  ***************************************************************************/
 
 #include "extendedpushbutton.h"
+#include "prokyon.h"
+#include "pixmapcache.h"
+
+ExtendedPushButton::ExtendedPushButton(QString label, QWidget *parent, const char *name):
+    QPushButton(label, parent, name)
+{
+    setPixmap(pixmapcache->get("action_selectall.png"));
+}
 
-ExtendedPushButton::ExtendedPushButton(QString label, QWidget *parent, const char *name): QPushButton(label, parent, name) {}
 ExtendedPushButton::~ExtendedPushButton() {}
 
 void ExtendedPushButton::mousePressEvent(QMouseEvent *e)
