@@ -22,9 +22,13 @@
 #include "configuration.h"
 #include "jobmanager.h"
 #include "job_deletetrackphysically.h"
-
 #include "lvi_track.h"
 #include "job_modifyplaylist_tracks.h"
+#include "job_getextradata.h"
+#include "pixmapcache.h"
+#ifdef HAVE_MIXXX
+#include "mixxxclient.h"
+#endif /* HAVE_MIXXX */
 
 #include <qlayout.h>
 #include <qhgroupbox.h>
@@ -36,6 +40,8 @@
 #include <qheader.h>
 #include <qpopupmenu.h>
 #include <qtooltip.h>
+#include <qmessagebox.h>
+#include <qstringlist.h>
 
 #include <stdlib.h>
 
@@ -75,7 +81,7 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   // checkboxes for additional informations
   QGroupBox *infobox = new QGroupBox(1, Qt::Horizontal, _("Additional Information"), hframe);
   QFrame *infoframe = new QFrame(infobox);
-  QGridLayout *infogrid = new QGridLayout(infoframe, 3, 3);
+  QGridLayout *infogrid = new QGridLayout(infoframe, 4, 3);
   
   info_lengthcheck = new QCheckBox(_("Length"), infoframe);
   infogrid->addWidget(info_lengthcheck, 0, 0);
@@ -91,10 +97,14 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   infogrid->addWidget(info_commentcheck, 1, 2);
   info_pathcheck = new QCheckBox(_("Path"), infoframe);
   infogrid->addWidget(info_pathcheck, 2, 0);	
-  info_filenamecheck = new QCheckBox("Filename", infoframe);
+  info_filenamecheck = new QCheckBox(_("Filename"), infoframe);
   infogrid->addWidget(info_filenamecheck, 2, 1);
   info_mediacheck = new QCheckBox(_("Medium"), infoframe);
   infogrid->addWidget(info_mediacheck, 2, 2);
+  info_notescheck = new QCheckBox(_("Notes"), infoframe);
+  infogrid->addWidget(info_notescheck, 3, 0);
+  info_ratingcheck = new QCheckBox(_("Rating"), infoframe);
+  infogrid->addWidget(info_ratingcheck, 3, 1);
   
   connect(info_lengthcheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Length(int)));
   connect(info_bitratecheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Bitrate(int)));
@@ -105,6 +115,8 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   connect(info_pathcheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Path(int)));
   connect(info_filenamecheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Filename(int)));
   connect(info_mediacheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Media(int)));
+  connect(info_notescheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Notes(int)));
+  connect(info_ratingcheck, SIGNAL(stateChanged(int)), this, SLOT(slot_Rating(int)));
 
   hlay->addWidget(infobox, 1);
   
@@ -126,56 +138,48 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   selblay->addWidget(hframe);
   selblay->addWidget(butframe, 1);
   
-  // tab search
-  searcher = new GUI_Searcher();
+  // external tabs
+  searcher   = new GUI_Searcher();
+  sql        = new GUI_SQL();
+#ifdef HAVE_MIXXX
+  mixxxQueue = new GUI_MixxxQueue();
+#endif /* HAVE_MIXXX */
   
   trackframe = new QFrame();
   trackframe->setFixedHeight(1);
 	
-  tabbar->addTab(trackframe, _("Tracks"));
+  tabbar->addTab(trackframe, pixmapcache->getFadeIconSet("music_sixteenthnote.png"), _("Tracks"));
 
-//========================================================================================
-//  tabbar->adjustSize();
-//  tabbarMinSize = tabbar->height() - 1;
-//  trackframe->setFixedHeight(0);
-
-//  tabbar->addTab(selectionframe, _("Selection"));
-//  tabbar->addTab(searcher, _("Search"));
-
-//  tabbar->adjustSize();
-//  tabbarMaxSize = tabbar->height();
-//  tabbar->setFixedHeight(tabbarMinSize);
-//----------------------------------------------------------------------------------------
-// ^^^ <-- adding of tabs 'Selection' and 'Search', sizing moved to polish() method [HoSa]
-//========================================================================================
-  
   connect(tabbar, SIGNAL(currentChanged(QWidget*)), this, SLOT(slot_tabChanged(QWidget*)));
   
   // the listing widget
   listing = new ExtendedListView(this);
-  char* temps = _( "Multiselect and right click to play titles,\nadd them to a playlist, or edit tags.\nYou can also drag and drop tracks to xmms." ); 
+  QString temps = _( "Multiselect and right click to play titles,\nadd them to a playlist, or edit tags.\nYou can also drag and drop tracks to xmms." ); 
   QToolTip::add( listing, temps);
   QToolTip::add( listing->viewport(), temps);
   listing->addColumn("", -1);
-  listing->addColumn(_("Medium"), 100);
-  listing->addColumn(_("Artist"), 120);
-  listing->addColumn(_("Title"), 160);
-  listing->addColumn(_("Length"), -1);
-  listing->setColumnAlignment(4, Qt::AlignRight);
-  listing->addColumn(_("Bitrate"), -1);
-  listing->setColumnAlignment(5, Qt::AlignRight);
-  listing->addColumn(_("Album"), 160);
-  listing->addColumn(_("Comment"), 160);
-  listing->addColumn(_("Year"), -1);
-  listing->setColumnAlignment(8, Qt::AlignCenter);
-  listing->addColumn(_("Genre"), 60);
-  listing->addColumn(_("Path"), 200);
-  listing->addColumn(_("Filename"), 200);
-  
+  columnMedium   = listing->addColumn(_("Medium"), 100);
+  columnArtist   = listing->addColumn(_("Artist"), 120);
+  columnTitle    = listing->addColumn(_("Title"), 160);
+  columnRating   = listing->addColumn(_("Rating"), 70 + fontMetrics().maxWidth());
+  columnLength   = listing->addColumn(_("Length"), -1);
+  listing->setColumnAlignment(columnLength, Qt::AlignRight);
+  columnBitrate  = listing->addColumn(_("Bitrate"), -1);
+  listing->setColumnAlignment(columnBitrate, Qt::AlignRight);
+  columnAlbum    = listing->addColumn(_("Album"), 160);
+  columnComment  = listing->addColumn(_("Comment"), 160);
+  columnYear     = listing->addColumn(_("Year"), -1);
+  listing->setColumnAlignment(columnYear, Qt::AlignCenter);
+  columnGenre    = listing->addColumn(_("Genre"), 60);
+  columnPath     = listing->addColumn(_("Path"), 200);
+  columnFilename = listing->addColumn(_("Filename"), 200);
+  columnNotes    = listing->addColumn(_("Notes"), 200);
+    
   listing->setItemMargin(0);
   
   listing->setAllColumnsShowFocus(true);
   listing->setSelectionMode(QListView::Extended);
+  listing->setShowSortIndicator(true);
   
   connect(listing, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
 	  this, SLOT(slot_contextMenu(QListViewItem*, const QPoint&, int)));
@@ -189,7 +193,8 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   blay->addWidget(tabbar);
   blay->addWidget(listing, 1);
   
-  for(int i = 0; i < 12; i++) widthes[i] = listing->columnWidth(i);
+  widthes.resize(listing->columns());
+  for(int i = 0; i < listing->columns(); i++) widthes[i] = listing->columnWidth(i);
   
   // set default states
   info_lengthcheck->setChecked( config->readInfo_Length() );
@@ -201,6 +206,8 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   info_genrecheck->setChecked( config->readInfo_Genre() );
   info_pathcheck->setChecked( config->readInfo_Path() );
   info_filenamecheck->setChecked( config->readInfo_Filename() );
+  info_notescheck->setChecked( config->readInfo_Notes() );
+  info_ratingcheck->setChecked( config->readInfo_Rating() );
   
   slot_Length( config->readInfo_Length() );
   slot_Media( config->readInfo_Media() );
@@ -211,6 +218,8 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
   slot_Genre( config->readInfo_Genre() );
   slot_Path( config->readInfo_Path() );
   slot_Filename( config->readInfo_Filename() );
+  slot_Notes( config->readInfo_Notes() );
+  slot_Rating( config->readInfo_Rating() );
   
   radioState = config->readinfo_radiostate();
   slot_changeRadio( radioState );
@@ -236,14 +245,15 @@ GUI_Listing::GUI_Listing(QWidget *parent, const char *name )
     for ( QStringList::Iterator it = qs.begin(); it != qs.end(); ++it ) {
       listing->header()->moveSection(loop,(*it).toInt());
       ++loop;
-    }
+    }  
   }
 
   listing->triggerUpdate();
 
   editmode = false;
-
-  resetSorting();
+#ifdef HAVE_MIXXX
+  mixxxMode = false;
+#endif /* HAVE_MIXXX */
 }
 
 void GUI_Listing::polish()
@@ -253,35 +263,65 @@ void GUI_Listing::polish()
 
   /* add tabs 'Selection' and 'Search' */
   /*-----------------------------------*/
-  tabbar->addTab(selectionframe, _("Selection"));
-  tabbar->addTab(searcher, _("Search"));
-    
+  tabbar->addTab(selectionframe, pixmapcache->getFadeIconSet("filter.png"), _("Selection"));
+  tabbar->addTab(searcher, pixmapcache->getFadeIconSet("action_updateonly.png"), _("Search"));
+  tabbar->addTab(sql, pixmapcache->getFadeIconSet("action_updateonly.png"), _("SQL"));
+
   /* determine maximum size of tabbar */
   /*----------------------------------*/
   tabbarMaxSize = tabbar->minimumSizeHint().height();
-    
+
+#ifdef HAVE_MIXXX
+  tabbar->addTab(mixxxQueue, pixmapcache->getFadeIconSet("mixxx.png"), _("Mixxx Queue"));
+#endif /* HAVE_MIXXX */
+
   /* set tabbar size for tab 'Tracks' */
   /*----------------------------------*/
   tabbar->setFixedHeight(tabbarMinSize);
 }
 
 void GUI_Listing::slot_tabChanged(QWidget *widget) {
-	if(widget != trackframe) tabbar->setFixedHeight(tabbarMaxSize);
-	else tabbar->setFixedHeight(tabbarMinSize);
+
+#ifdef HAVE_MIXXX
+    if(widget == mixxxQueue) {
+        mixxxMode = true;
+        gui->enterMixxxMode();
+        tabbar->setMinimumHeight(tabbarMaxSize);
+        tabbar->setMaximumHeight(listing->maximumHeight());
+    } else {
+        if(mixxxMode) {
+            mixxxMode = false;
+            gui->leaveMixxxMode();
+        }
+#endif /* HAVE_MIXXX */
+
+        if(widget != trackframe) {
+            // set tabbar to polished size
+            tabbar->setFixedHeight(tabbarMaxSize);
+        } else {
+            // hide tabbar
+            tabbar->setFixedHeight(tabbarMinSize);
+        }
+#ifdef HAVE_MIXXX
+    }
+#endif /* HAVE_MIXXX */
 }
 
 // ##############################################
 // # callbacks for the checkboxes
 // ##############################################
-void GUI_Listing::slot_Media(int state)    { toggleColumn(1, state); }
-void GUI_Listing::slot_Length(int state)   { toggleColumn(4, state); }
-void GUI_Listing::slot_Bitrate(int state)  { toggleColumn(5, state); }
-void GUI_Listing::slot_Album(int state)    { toggleColumn(6, state); }
-void GUI_Listing::slot_Comment(int state)  { toggleColumn(7, state); }
-void GUI_Listing::slot_Year(int state)     { toggleColumn(8, state); }
-void GUI_Listing::slot_Genre(int state)    { toggleColumn(9, state); }
-void GUI_Listing::slot_Path(int state)     { toggleColumn(10, state);}
-void GUI_Listing::slot_Filename(int state) { toggleColumn(11, state);}
+// always show columnArtist and columnTitle
+void GUI_Listing::slot_Media(int state)    { toggleColumn(columnMedium, state); }
+void GUI_Listing::slot_Length(int state)   { toggleColumn(columnLength, state); }
+void GUI_Listing::slot_Bitrate(int state)  { toggleColumn(columnBitrate, state); }
+void GUI_Listing::slot_Album(int state)    { toggleColumn(columnAlbum, state); }
+void GUI_Listing::slot_Comment(int state)  { toggleColumn(columnComment, state); }
+void GUI_Listing::slot_Year(int state)     { toggleColumn(columnYear, state); }
+void GUI_Listing::slot_Genre(int state)    { toggleColumn(columnGenre, state); }
+void GUI_Listing::slot_Path(int state)     { toggleColumn(columnPath, state);}
+void GUI_Listing::slot_Filename(int state) { toggleColumn(columnFilename, state);}
+void GUI_Listing::slot_Notes(int state)    { toggleColumn(columnNotes, state);}
+void GUI_Listing::slot_Rating(int state)   { toggleColumn(columnRating, state);}
 
 // ##############################################
 // # callbacks for the buttons
@@ -306,8 +346,6 @@ void GUI_Listing::slot_doubleClicked(QListViewItem *lvi) {
 }
 
 void GUI_Listing::slot_selectionChanged2(int c) {
-  if(c == lastsort) sortlist[lastsort] *= -1;
-  else sortlist[lastsort = c] = ++sortcount;
   if(!editmode) gui->getTagListing()->notifySelectionChanged();
   else gui->getTagListing()->notifySortingInEditModeChanged();
 }
@@ -324,26 +362,43 @@ void GUI_Listing::slot_contextMenu(QListViewItem *lvi, const QPoint &p, int)
   if(editmode) return;
 
   QPopupMenu *menu = new QPopupMenu(this);
-//-  menu->insertTitle("Track Menu");
 
   LVI_Track *item = static_cast<LVI_Track*>(lvi);
-
-//   if(item->getType() != MEDIUM_HARDDISK) {
-//     if(!item->getIsRequested()) menu->insertItem("Request Track", this, SLOT(slot_requestTrack()), 0, 1);
-//     else menu->insertItem("Delete Track Requestion", this, SLOT(slot_unrequestTrack()), 0, 1);
-//     if(busyMap[item->getMedium()]) menu->disableItem(1);
-//   }
-
+  
+  int i=0;
   if(item->getIsAvailable()) {
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_xmms.png"), _("PLAY track(s)"), this, SLOT(slot_playWithXMMS()), 0, 2);
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_xmms.png"), _("ENQUEUE track(s)"), this, SLOT(slot_enqueueInXMMS()), 0, 3);
+#ifdef HAVE_MIXXX
+    if(config->RegularPlayerEnabled()) {
+#endif /* HAVE_MIXXX */
+      menu->insertItem(pixmapcache->get("stock_media-play.png"), _("Play track(s) with regular player"), this, SLOT(slot_playWithXMMS()), 0, i++);
+      menu->insertItem(pixmapcache->get("stock_music-library.png"), _("Enqueue track(s) at regular player"), this, SLOT(slot_enqueueInXMMS()), 0, i++);
+      menu->insertSeparator();
+#ifdef HAVE_MIXXX
+    }
+    if(config->MixxxPlayerEnabled() && isMixxxConnected) {
+      menu->insertItem(pixmapcache->get("play-off-Ch1.png"), _("mixxx: load at channel 1"), this, SLOT(slot_mixxxLoadAtChannel1()), 0, i++);
+      menu->insertItem(pixmapcache->get("play-off-Ch2.png"), _("mixxx: load at channel 2"), this, SLOT(slot_mixxxLoadAtChannel2()), 0, i++);
+      menu->insertItem(pixmapcache->get("play-on-Ch1.png"), _("mixxx: load at channel 1 and play"), this, SLOT(slot_mixxxLoadAndPlayAtChannel1()), 0, i++);
+      menu->insertItem(pixmapcache->get("play-on-Ch2.png"), _("mixxx: load at channel 2 and play"), this, SLOT(slot_mixxxLoadAndPlayAtChannel2()), 0, i++);
+      menu->insertItem(pixmapcache->get("stock_music-library.png"), _("mixxx: enqueue at channel 1"), this, SLOT(slot_mixxxEnqueueAtChannel1()), 0, i++);
+      menu->insertItem(pixmapcache->get("stock_music-library.png"), _("mixxx: enqueue at channel 2"), this, SLOT(slot_mixxxEnqueueAtChannel2()), 0, i++);
+      menu->insertSeparator();
+    }
+#endif /* HAVE_MIXXX */
+    
   }
-//    menu->insertItem("append track(s) to current", this, SLOT(slot_addToCurrent()), 0, 5);
-    if(gui->getPlaylisting()->getDisplayedPlaylistID() > 0) menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_playlist.png"), _("append track(s) to selected playlist"), this, SLOT(slot_addToPlaylist()), 0, 6);
-  if(item->getIsAvailable() && item->getType() != MEDIUM_CDROM) {
+
+  if(gui->getPlaylisting()->getDisplayedPlaylistID() > 0) {
+    menu->insertItem(pixmapcache->get("lvi_playlist.png"), _("append track(s) to selected playlist"), this, SLOT(slot_addToPlaylist()), 0, i++);
     menu->insertSeparator();
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_remove.png"), _("Delete Track Physically"), this, SLOT(slot_deleteTrack()), 0, 4);
-    if(busyMap.contains(item->getMedium()) && busyMap[item->getMedium()]) menu->setItemEnabled(4, false);
+  }
+
+  menu->insertItem(pixmapcache->get("lvi_changed.png"), _("Edit Selected Track(s)"), gui->getTagListing(), SLOT(slot_editbutton()), 0, i++);
+  menu->insertSeparator();
+
+  if(item->getIsAvailable() && item->getType() != MEDIUM_CDROM) {
+    menu->insertItem(pixmapcache->get("action_remove.png"), _("Delete Track Physically"), this, SLOT(slot_deleteTrack()), 0, i);
+    if(busyMap.contains(item->getMedium()) && busyMap[item->getMedium()]) menu->setItemEnabled(i, false);
   }
 
   menu->exec(p, 0);
@@ -351,61 +406,122 @@ void GUI_Listing::slot_contextMenu(QListViewItem *lvi, const QPoint &p, int)
 
 void GUI_Listing::slot_deleteTrack()
 {
-  LVI_Track *item = static_cast<LVI_Track*>(listing->currentItem());
+  LVI_Track *lvi;
+  for (QListViewItem *item = listing->firstChild(); item != 0; item = item->nextSibling())
+    if ( item->isSelected() && ( lvi = dynamic_cast<LVI_Track*>(item))) 
+    {
+      TRACK *track = new TRACK();
+      track->id     = lvi->getOriginalTrack()->id;
+      track->artist = lvi->getOriginalTrack()->artist;
+      track->path   = lvi->getOriginalTrack()->path;
+      track->filename = lvi->getOriginalTrack()->filename;
+      track->medium = lvi->getOriginalTrack()->medium;
 
-  TRACK *track = new TRACK();
-  track->id     = item->getOriginalTrack()->id;
-  track->artist = item->getOriginalTrack()->artist;
-  track->path   = item->getOriginalTrack()->path;
-  track->filename = item->getOriginalTrack()->filename;
-  track->medium = item->getOriginalTrack()->medium;
-  //  track->isRequested = item->getOriginalTrack()->isRequested;
-
-  jobman->addJob(new Job_DeleteTrackPhysically(track));
+      jobman->addJob(new Job_DeleteTrackPhysically(track));
+    }
 }
 
+
 // ##############################################
 // # callback for context menu items
 // ##############################################
-QString GUI_Listing::adjustString(QString s)
+
+void GUI_Listing::slot_playWithXMMS(bool selected) {
+  callRegularPlayer(selected, false);
+}
+
+void GUI_Listing::slot_enqueueInXMMS(bool selected) {
+  callRegularPlayer(selected, true);
+}
+
+void GUI_Listing::callRegularPlayer(bool selected, bool enqueue)
 {
-  for(unsigned i=0; i<s.length(); i++) {
-    if(s[i] == '\"' || s[i] == '\\') s.insert(i++, '\\');
-  }
-  return s;
-}
-
-void GUI_Listing::slot_playWithXMMS() {
-  QString command;
-  QString tmpCmd;
-  
-  command.sprintf("%s", config->getPlayCmd().stripWhiteSpace().latin1() );
+  QStringList fileList;
   
   for (QListViewItem *item = listing->firstChild(); item != 0; item = item->nextSibling())
   {
-     if (item->isSelected() && dynamic_cast<LVI_Track*>(item)->getIsAvailable())
-        command.append( tmpCmd.sprintf(" \"%s\"", (const char*)adjustString(QString(item->text(10) + "/" + item->text(11)).stripWhiteSpace().utf8())) );
+     if ( ( !selected || item->isSelected()) && dynamic_cast<LVI_Track*>(item)->getIsAvailable() )
+         fileList.append(QString(item->text(columnPath) + "/" + item->text(columnFilename)));
   }
-  command.append(" &");
-  //  qWarning ( "system: %s", command.latin1() );
-  system(command.latin1());
+  
+  commandLineCallToRegularPlayer(fileList, enqueue);
 }
 
-void GUI_Listing::slot_enqueueInXMMS() {
-  QString command;
-  QString tmpCmd;
-  
-  command.sprintf("%s", config->getEnqueueCmd().stripWhiteSpace().latin1() );
+void GUI_Listing::slot_mixxxLoadAtChannel1() {
+#ifdef HAVE_MIXXX
+  mixxxLoadAtChannel(1);
+#endif /* HAVE_MIXXX */
+}
+
+void GUI_Listing::slot_mixxxLoadAtChannel2() {
+#ifdef HAVE_MIXXX
+  mixxxLoadAtChannel(2);
+#endif /* HAVE_MIXXX */
+}
+
+#ifdef HAVE_MIXXX
+void GUI_Listing::mixxxLoadAtChannel(int channel)
+{
+  QListViewItem *item = listing->currentItem ();
+  if (dynamic_cast<LVI_Track*>(item)->getIsAvailable() ) {
+    mixxxClient->loadAtChannel(QString(item->text(columnPath) + "/" + item->text(columnFilename)), channel);
+  } else {
+    QMessageBox::information(this, "Message", _("File(s) not available for playing"), QMessageBox::Ok);
+  }
+}
+#endif /* HAVE_MIXXX */
+
+void GUI_Listing::slot_mixxxLoadAndPlayAtChannel1() {
+#ifdef HAVE_MIXXX
+  mixxxLoadAndPlayAtChannel(1);
+#endif /* HAVE_MIXXX */
+}
+
+void GUI_Listing::slot_mixxxLoadAndPlayAtChannel2() {
+#ifdef HAVE_MIXXX
+  mixxxLoadAndPlayAtChannel(2);
+#endif /* HAVE_MIXXX */
+}
+
+#ifdef HAVE_MIXXX
+void GUI_Listing::mixxxLoadAndPlayAtChannel(int channel)
+{
+  QListViewItem *item = listing->currentItem ();
+  if (dynamic_cast<LVI_Track*>(item)->getIsAvailable() ) {
+    mixxxClient->loadAndPlayAtChannel(QString(item->text(columnPath) + "/" + item->text(columnFilename)), channel);
+  } else {
+    QMessageBox::information(this, "Message", _("File(s) not available for playing"), QMessageBox::Ok);
+  }
+}
+#endif /* HAVE_MIXXX */
+
+void GUI_Listing::slot_mixxxEnqueueAtChannel1() {
+#ifdef HAVE_MIXXX
+  mixxxEnqueueAtChannel(true, 1);
+#endif /* HAVE_MIXXX */
+}
+
+void GUI_Listing::slot_mixxxEnqueueAtChannel2() {
+#ifdef HAVE_MIXXX
+  mixxxEnqueueAtChannel(true, 2);
+#endif /* HAVE_MIXXX */
+}
+
+#ifdef HAVE_MIXXX
+void GUI_Listing::mixxxEnqueueAtChannel(bool selected, int channel)
+{
+  QList<LVI_Track> list;
   
   for (QListViewItem *item = listing->firstChild(); item != 0; item = item->nextSibling())
   {
-     if (item->isSelected() && dynamic_cast<LVI_Track*>(item)->getIsAvailable())
-  	     command.append( tmpCmd.sprintf(" \"%s\"", (const char*)adjustString(QString(item->text(10) + "/" + item->text(11)).stripWhiteSpace().utf8())) );
+     if ( ( !selected || item->isSelected()) && dynamic_cast<LVI_Track*>(item)->getIsAvailable() ) {
+         list.append(dynamic_cast<LVI_Track*>(item));
+     }
   }
-  command.append(" &");
-//	cout << command << endl;
-  system(command.latin1());
+  
+  mixxxQueue->enqueueAtChannel(list, channel);
 }
+#endif /* HAVE_MIXXX */
 
 void GUI_Listing::slot_addToCurrent() {
   QList<TRACK> *templist;
@@ -486,15 +602,17 @@ void GUI_Listing::toggleColumn(int column, int state)
     listing->triggerUpdate();
     // ^^^ you need this instead of listing->setColumnWidth(column, widthes[column]) ==> BUG?! in Qt
     
-    if(column == 1)       listing->setColumnText(1, _("Medium"));
-    else if(column == 4)  listing->setColumnText(4, _("Length"));
-    else if(column == 5)  listing->setColumnText(5, _("Bitrate"));
-    else if(column == 6)  listing->setColumnText(6, _("Album"));
-    else if(column == 7)  listing->setColumnText(7, _("Comment"));
-    else if(column == 8)  listing->setColumnText(8, _("Year"));
-    else if(column == 9)  listing->setColumnText(9, _("Genre"));
-    else if(column == 10) listing->setColumnText(10, _("Path"));
-    else if(column == 11) listing->setColumnText(11, _("Filename"));
+    if(column == columnMedium)        listing->setColumnText(columnMedium,   _("Medium"));
+    else if(column == columnLength)   listing->setColumnText(columnLength,   _("Length"));
+    else if(column == columnBitrate)  listing->setColumnText(columnBitrate,  _("Bitrate"));
+    else if(column == columnAlbum)    listing->setColumnText(columnAlbum,    _("Album"));
+    else if(column == columnComment)  listing->setColumnText(columnComment,  _("Comment"));
+    else if(column == columnYear)     listing->setColumnText(columnYear,     _("Year"));
+    else if(column == columnGenre)    listing->setColumnText(columnGenre,    _("Genre"));
+    else if(column == columnPath)     listing->setColumnText(columnPath,     _("Path"));
+    else if(column == columnFilename) listing->setColumnText(columnFilename, _("Filename"));
+    else if(column == columnNotes)    listing->setColumnText(columnNotes,    _("Notes"));
+    else if(column == columnRating)   listing->setColumnText(columnRating,   _("Rating"));
     
     if ( config->getMaxWidthMode() ) 
 	       listing->setColumnWidthMode(column, QListView::Maximum);
@@ -531,7 +649,6 @@ void GUI_Listing::notifyMediumAvailabilityChanged(int id, QString path, bool sta
       if(c > 0) {
 	// restore sorting;
 	sort();
-	
 	if(listing->childCount() != 0) listing->setSelected(listing->firstChild(), true);
 	slot_selectionChanged();
       }
@@ -550,7 +667,7 @@ void GUI_Listing::notifyNewTrackListing(QList<TRACK> *list)
   if(editmode) return;
   
   // clear old data
-  listing->clear();        // DEBUG ??? delete every single LVI ???
+  listing->clear();    
   listAll.clear();
 
   if(list != 0) for(TRACK *entry = list->first(); entry != 0; entry = list->next()) {
@@ -586,6 +703,11 @@ void GUI_Listing::notifyMediumRemoved(int id)
   mapAvail.remove(id);
 }
 
+void GUI_Listing::notifyMediumRenamed(MEDIUM *medium)
+{
+  mapLabel[medium->id] = medium->label;
+}
+
 void GUI_Listing::notifyFilenameHasChanged(int id, QString filename) {
   for(LVI_Track *item = listAll.first(); item != 0; item = listAll.next())
     if(item->getOriginalTrack()->id == id) item->setNewFilename(filename);
@@ -600,7 +722,7 @@ void GUI_Listing::setEditMode(bool state)
     oldtip = QToolTip::textFor( listing, QPoint(0,0));
     QToolTip::remove( listing );
     QToolTip::remove( listing->viewport() );
-    char* temps = _( "Select one track to edit.\nAll modifications will be flagged,\nand need to be saved" );
+    QString temps = _( "Select one track to edit.\nAll modifications will be flagged,\nand need to be saved" );
     QToolTip::add( listing, temps );
     QToolTip::add( listing->viewport(), temps );
   } else { // restore previous tooltip
@@ -610,12 +732,15 @@ void GUI_Listing::setEditMode(bool state)
     QToolTip::add( listing->viewport(), oldtip);
   }
 
-  
+#ifdef HAVE_MIXXX
+  tabbar->setTabEnabled(mixxxQueue, !editmode);
+#endif /* HAVE_MIXXX */
   button_All->setEnabled(!editmode);
   button_None->setEnabled(!editmode);
   button_Invert->setEnabled(!editmode);
   selbox->setEnabled(!editmode);
-  searcher->setConnectionState(!editmode);
+  searcher->slot_setConnectionState(!editmode);
+  sql->slot_setConnectionState(!editmode);
 
   if(editmode) datadispatcher->eventAddReadWriteLock();
   else datadispatcher->eventRemoveReadWriteLock();
@@ -646,39 +771,51 @@ void GUI_Listing::setEditMode(bool state)
   if ( verbose == 4 ) qWarning( "GUI_Listing::setEditMode() ends");
 }
 
-void GUI_Listing::resetSorting()
-{
-  for(int i=0; i<12; i++) sortlist[i] = 0;
-  sortlist[2] = 2;
-  sortlist[3] = 1;
 
-  sortcount = 2;
-  lastsort = 2;
+int GUI_Listing::getHeaderIndex( QString s) {
+  // return colum index whose header name is s
+  // Column position is fully user configurable
+  // Index column will not change however.
+  int i;
+  QHeader* qh( listing->header() );
+  for (i = 0 ; i < listing->header()->count(); i++) 
+    if ( qh->label(qh->mapToSection(i)) == s ) return i;
+  return 999;   // not found, do not sort on this index
 }
 
 void GUI_Listing::sort()
 {
-  int min = 0, actmax, actindex;
+  int i;
+  LVI_Track *item = static_cast<LVI_Track*>(listing->firstChild());
+  if (config->getComboSortingAuto() && item && item->getTracknumber() ) 
+    i = getHeaderIndex( _("Album") );
+  else 
+    i = getHeaderIndex(  config->getComboSortingMode() );
 
-  do {
-    actmax = sortcount;
-    for(int i=0; i<12; i++) if(abs(sortlist[i]) > min && abs(sortlist[i]) <= actmax) { actindex = i; actmax = abs(sortlist[i]); }
-    listing->setSorting(actindex, (abs(sortlist[actindex]) > 0));
-    listing->sort();
-    min = actmax;
-  } while(actmax != sortcount);
+  // if index not found then use filename as a default.
+  if ( i== 999 )  i = getHeaderIndex( _("Filename") );
+
+  listing->setSorting( listing->header()->mapToSection(i), true); 
+
+  //  qWarning(" sorting on idx:%d sec:%d title:%s", i, listing->header()->mapToSection(i),
+  //listing->header()->label(listing->header()->mapToSection(i)).local8Bit().data());
 }
 
-
-void GUI_Listing::setConnectionState(bool state)
+void GUI_Listing::slot_setConnectionState(bool state)
 {
-  searcher->setConnectionState(state);
   if(state == false) clear();
   if(state == false) editmode = false;
   isConnected = state;
-
 }
 
+#ifdef HAVE_MIXXX
+void GUI_Listing::setMixxxConnectionState(bool state)
+{
+  mixxxQueue->setMixxxConnectionState(state);
+  isMixxxConnected = state;
+}
+#endif /* HAVE_MIXXX */
+
 QStringList GUI_Listing::readColumnWidth()
 {
   QStringList qs;
