@@ -16,16 +16,6 @@
  ***************************************************************************/
 
 #include "gui_sourcer.h"
-
-#include <qpixmap.h>
-#include <qfile.h>
-#include <qdir.h>
-#include <qtextstream.h>
-
-#include <qpopupmenu.h>
-#include <qtooltip.h>
-#include <qmessagebox.h>
-
 #include "gui.h"
 #include "jobmanager.h"
 #include "datadispatcher.h"
@@ -34,11 +24,19 @@
 #include "job_appendcdromtodatabase.h"
 #include "job_appendsmbtodatabase.h"
 #include "job_appendnfstodatabase.h"
+#include "mtabwatcher.h"
+#include "supermountwatcher.h"
+#include "submountwatcher.h"
+#include "pixmapcache.h"
 
-#include "dirwatcher.h"
+#include <qfile.h>
+#include <qdir.h>
+#include <qtextstream.h>
+#include <qregexp.h>
+#include <qpopupmenu.h>
+#include <qtooltip.h>
+#include <qmessagebox.h>
 
-#define FSTAB     "/etc/fstab"
-#define MTAB      "/etc/mtab"
 #define DELIMITER '#'
 #define BLANK     ' '
 
@@ -58,7 +56,7 @@ GUI_Sourcer::GUI_Sourcer(QWidget *parent, const char *name )
   
   // item for local files
   lviHDD = new QListViewItem(this, _("Files on Harddisk"));
-  lviHDD->setPixmap(0, QPixmap(PROKYON_HOME + "/images/lvi_hdd.png"));
+  lviHDD->setPixmap(0, pixmapcache->get("lvi_hdd.png"));
   
   connect(this, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
 	  this, SLOT(slot_contextMenu(QListViewItem*, const QPoint&, int)));
@@ -79,18 +77,18 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
   if(isConnected && item == lviHDD && !doSynchro) {
     QPopupMenu *menu = new QPopupMenu(this);
 //-    menu->insertTitle("Harddisk Operations");
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize6()), 0, 6);
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize7()), 0, 7);
+    menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize6()), 0, 6);
+    menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize7()), 0, 7);
     menu->insertSeparator();
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize2()), 0, 2);
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize3()), 0, 3);
+    menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize2()), 0, 2);
+    menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize3()), 0, 3);
     menu->insertSeparator();
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize4()), 0, 4);
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize5()), 0, 5);
+    menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize4()), 0, 4);
+    menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize5()), 0, 5);
     menu->insertSeparator();
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize1()), 0, 1);
+    menu->insertItem(pixmapcache->get("action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize1()), 0, 1);
     menu->insertSeparator();
-    menu->insertItem(QPixmap(PROKYON_HOME + "/images/help.png"), _("More Info"), this, SLOT(slot_menuHelp()), 0, 8);
+    menu->insertItem(pixmapcache->get("help.png"), _("More Info"), this, SLOT(slot_menuHelp()), 0, 8);
     if(rwLock > 0) {
       menu->setItemEnabled(1, false);
       menu->setItemEnabled(2, false);
@@ -109,24 +107,24 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
     QPopupMenu *menu = new QPopupMenu(this);
 //-    menu->insertTitle("SMB Share Operations");
     if(lvi->getIsMounted()) {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_smb.png"), _("Unmount"), this, SLOT(slot_menuUnmountSMB()), 0, 1);
+      menu->insertItem(pixmapcache->get("lvi_smb.png"), _("Unmount"), this, SLOT(slot_menuUnmountSMB()), 0, 1);
       if(isConnected) {
 	menu->insertSeparator();
       	if(!lvi->getIsKnown()) {
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendSMB()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendSMB()), 0, 2);
 	  if(rwLock > 0) menu->setItemEnabled(2, false);
 	} else {
 	  currSMB = lvi;
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize16()), 0, 7);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize17()), 0, 8);
+	  menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize16()), 0, 7);
+	  menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize17()), 0, 8);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize12()), 0, 3);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize13()), 0, 4);
+	  menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize12()), 0, 3);
+	  menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize13()), 0, 4);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize14()), 0, 5);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize15()), 0, 6);
+	  menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize14()), 0, 5);
+	  menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize15()), 0, 6);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize11()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize11()), 0, 2);
 	  if(rwLock > 0) {
 	    menu->setItemEnabled(2, false);
 	    menu->setItemEnabled(3, false);
@@ -139,7 +137,7 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
 	}
       }
     } else {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_smb.png"), _("Mount"), this, SLOT(slot_menuMountSMB()), 0, 1);
+      menu->insertItem(pixmapcache->get("lvi_smb.png"), _("Mount"), this, SLOT(slot_menuMountSMB()), 0, 1);
     }
     menu->exec(point, 0);
     delete menu;
@@ -151,24 +149,24 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
     QPopupMenu *menu = new QPopupMenu(this);
 //-    menu->insertTitle("NFS Share Operations");
     if(lvi->getIsMounted()) {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_nfs.png"), _("Unmount"), this, SLOT(slot_menuUnmountNFS()), 0, 1);
+      menu->insertItem(pixmapcache->get("lvi_nfs.png"), _("Unmount"), this, SLOT(slot_menuUnmountNFS()), 0, 1);
       if(isConnected) {
 	menu->insertSeparator();
       	if(!lvi->getIsKnown()) {
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendNFS()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendNFS()), 0, 2);
 	  if(rwLock > 0) menu->setItemEnabled(2, false);
 	} else {
 	  currNFS = lvi;
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize36()), 0, 7);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize37()), 0, 8);
+	  menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Database)"), this, SLOT(slot_menuSynchronize36()), 0, 7);
+	  menu->insertItem(pixmapcache->get("action_synchronize.png"), _("Synchronize (prefer Sources)"), this, SLOT(slot_menuSynchronize37()), 0, 8);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize32()), 0, 3);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize33()), 0, 4);
+	  menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags"), this, SLOT(slot_menuSynchronize32()), 0, 3);
+	  menu->insertItem(pixmapcache->get("action_readtags.png"), _("Update + Read Tags (force)"), this, SLOT(slot_menuSynchronize33()), 0, 4);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize34()), 0, 5);
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize35()), 0, 6);
+	  menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags"), this, SLOT(slot_menuSynchronize34()), 0, 5);
+	  menu->insertItem(pixmapcache->get("action_writetags.png"), _("Update + Write Tags (force)"), this, SLOT(slot_menuSynchronize35()), 0, 6);
 	  menu->insertSeparator();
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize31()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize31()), 0, 2);
 	  if(rwLock > 0) {
 	    menu->setItemEnabled(2, false);
 	    menu->setItemEnabled(3, false);
@@ -181,7 +179,7 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
 	}
       }
     } else {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_nfs.png"), _("Mount"), this, SLOT(slot_menuMountNFS()), 0, 1);
+      menu->insertItem(pixmapcache->get("lvi_nfs.png"), _("Mount"), this, SLOT(slot_menuMountNFS()), 0, 1);
     }
     menu->exec(point, 0);
     delete menu;
@@ -194,24 +192,28 @@ void GUI_Sourcer::slot_contextMenu(QListViewItem* item, const QPoint &point, int
     QPopupMenu *menu = new QPopupMenu(this);
 //-    menu->insertTitle("CDROM Operations");
     if(lvi->getIsMounted()) {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_cdrom.png"), _("Unmount"), this, SLOT(slot_menuUnmountCDROM()), 0, 1);
+      if(lvi->getIsAutoMounted() == SUBMOUNT) {
+        menu->insertItem(pixmapcache->get("lvi_cdrom.png"), _("Eject"), this, SLOT(slot_menuUnmountCDROM()), 0, 1);
+      } else if(lvi->getIsAutoMounted() == NO_AUTOMOUNT) {
+        menu->insertItem(pixmapcache->get("lvi_cdrom.png"), _("Unmount"), this, SLOT(slot_menuUnmountCDROM()), 0, 1);
+      }
       if(isConnected) {
 	menu->insertSeparator();
 	if(!lvi->getIsKnown()) {
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendCDROM()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_append.png"), _("Append To Database"), this, SLOT(slot_menuAppendCDROM()), 0, 2);
 	  if(rwLock > 0) menu->setItemEnabled(2, false);
 	} else {
 	  currCDROM = lvi;
-	  menu->insertItem(QPixmap(PROKYON_HOME + "/images/action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize21()), 0, 2);
+	  menu->insertItem(pixmapcache->get("action_updateonly.png"), _("Update Only"), this, SLOT(slot_menuSynchronize21()), 0, 2);
 	  if(rwLock > 0) {
 	    menu->setItemEnabled(2, false);
 	  }
 	}
       }
-    } else {
-      menu->insertItem(QPixmap(PROKYON_HOME + "/images/lvi_cdrom.png"), _("Mount"), this, SLOT(slot_menuMountCDROM()), 0, 1);
+    } else if(lvi->getIsAutoMounted() != SUBMOUNT) {
+      menu->insertItem(pixmapcache->get("lvi_cdrom.png"), _("Mount"), this, SLOT(slot_menuMountCDROM()), 0, 1);
     }
-    menu->exec(point, 0);
+    if(menu->idAt(0) != -1) menu->exec(point, 0); // at least one entry
     delete menu;
   }
 }
@@ -301,43 +303,78 @@ void GUI_Sourcer::init()
 	s = s.remove(0, s.find(BLANK) + 1);
 	QString mountoptions = s.left(s.find(BLANK));
 	s = s.remove(0, s.find(BLANK) + 1);
-	if(fstype.find("smbfs", 0, false) != -1 && (devicename.find("mp3", 0, false) != -1   || mountpoint.find("mp3", 0, false) != -1 ||
-                                                devicename.find("ogg", 0, false) != -1   || mountpoint.find("ogg", 0, false) != -1 ||
-                                                devicename.find("data", 0, false) != -1  || mountpoint.find("data", 0, false) != -1 ||
-                                                devicename.find("music", 0, false) != -1 || mountpoint.find("music", 0, false) != -1 ||
-                                                devicename.find("audio", 0, false) != -1 || mountpoint.find("audio", 0, false) != -1))
-    {
-	  smbcount++;
-	  LVI_SMBShare *smb = new LVI_SMBShare(this, smbcount, mountpoint, devicename);
-	  smb->moveItem(prev);
-	  prev = smb;
-	  mapSMB[smbcount] = smb;
-	  if(isPathMounted(mountpoint)) smb->applyMount();
+	if(fstype.find("smbfs", 0, false) != -1) {
+        bool keywordMatch = false;
+        QStringList smbKeywords = config->getSmbKeywords();
+        if(smbKeywords.isEmpty()) {
+            // show all amb shares
+            keywordMatch = true;
+        } else {
+            for(QStringList::Iterator it = smbKeywords.begin(); it != smbKeywords.end(); ++it ) {
+                if(devicename.find(*it, 0, false) != -1 || mountpoint.find(*it, 0, false) != -1) {
+                    keywordMatch = true;
+                    break;
+                }
+            }
+        }
+        if(keywordMatch) {
+	        smbcount++;
+	        LVI_SMBShare *smb = new LVI_SMBShare(this, smbcount, mountpoint, devicename);
+	        smb->moveItem(prev);
+	        prev = smb;
+	        mapSMB[smbcount] = smb;
+	        if(isPathMounted(mountpoint)) smb->applyMount();
+        }
 	}
-	else if(fstype.find("nfs", 0, false) != -1 && (devicename.find("mp3", 0, false) != -1 || mountpoint.find("mp3", 0, false) != -1 ||
-                                                   devicename.find("ogg", 0, false) != -1   || mountpoint.find("ogg", 0, false) != -1 ||
-                                                   devicename.find("data", 0, false) != -1  || mountpoint.find("data", 0, false) != -1 ||
-                                                   devicename.find("music", 0, false) != -1 || mountpoint.find("music", 0, false) != -1 ||
-                                                   devicename.find("audio", 0, false) != -1 || mountpoint.find("audio", 0, false) != -1))
-    {
-	  nfscount++;
-	  LVI_NFSExport *nfs = new LVI_NFSExport(this, nfscount, mountpoint, devicename);
-	  nfs->moveItem(prev);
-	  prev = nfs;
-	  mapNFS[nfscount] = nfs;
-	  if(isPathMounted(mountpoint)) nfs->applyMount();
+	else if(fstype.find("nfs", 0, false) != -1) {
+        bool keywordMatch = false;
+        QStringList nfsKeywords = config->getNfsKeywords();
+        if(nfsKeywords.isEmpty()) {
+            // show all amb shares
+            keywordMatch = true;
+        } else {
+            for(QStringList::Iterator it = nfsKeywords.begin(); it != nfsKeywords.end(); ++it ) {
+                if(devicename.find(*it, 0, false) != -1 || mountpoint.find(*it, 0, false) != -1) {
+                    keywordMatch = true;
+                    break;
+                }
+            }
+        }
+        if(keywordMatch) {
+	        nfscount++;
+	        LVI_NFSExport *nfs = new LVI_NFSExport(this, nfscount, mountpoint, devicename);
+	        nfs->moveItem(prev);
+	        prev = nfs;
+	        mapNFS[nfscount] = nfs;
+	        if(isPathMounted(mountpoint)) nfs->applyMount();
+        }
 	}
-	else if(devicename.find("cdrom", 0, false)      != -1 || mountpoint.find("cdrom", 0, false) != -1 ||
-		devicename.find("dvd", 0, false)        != -1 || mountpoint.find("dvd", 0, false)   != -1 ||
-		devicename.find("cdrecorder", 0, false) != -1 || mountpoint.find("cdrecorder", 0, false) != -1)
+	else if(devicename.find("cdrom", 0, false)       != -1 || mountpoint.find("cdrom", 0, false)       != -1 ||
+		devicename.find("dvd", 0, false)         != -1 || mountpoint.find("dvd", 0, false)         != -1 ||
+		devicename.find("cdrecorder", 0, false)  != -1 || mountpoint.find("cdrecorder", 0, false)  != -1 ||
+		devicename.find("dvdrecorder", 0, false) != -1 || mountpoint.find("dvdrecorder", 0, false) != -1 )
+	  //devicename.find("floppy", 0, false) != -1 || mountpoint.find("floppy", 0, false) != -1)
 	{
 	  count++;
-	  int supermounted = (fstype.find("supermount",0,false) != -1);
-	  LVI_CDDrive *lvi = new LVI_CDDrive(this, count, mountpoint, devicename, supermounted);
-	  lvi->moveItem(prev);
-	  prev = lvi;
-	  mapCDROM[count] = lvi;
-	  if(isPathMounted(mountpoint)) jobman->addJob(new Job_ComputeCDROMID(lvi));
+     int automounted = NO_AUTOMOUNT;
+     if(fstype.find("supermount",0,false) != -1) {
+         // Mandrake 10
+         QRegExp rx("dev=([^,\\s]*)");
+         if(rx.search(mountoptions) > -1) {
+           devicename = rx.cap(1);
+         }
+         automounted = SUPERMOUNT;
+     } else if(fstype.find("subfs",0,false) != -1) {
+         automounted = SUBMOUNT;
+     }
+     
+     LVI_CDDrive *lvi = new LVI_CDDrive(this, count, mountpoint, devicename, automounted);
+     lvi->moveItem(prev);
+       prev = lvi;
+        mapCDROM[count] = lvi;
+    if((automounted && isPathAutoMounted(mountpoint)) ||
+      (!automounted && isPathMounted(mountpoint)))
+               jobman->addJob(new Job_ComputeCDROMID(lvi));
 	}
       }
     }
@@ -345,17 +382,21 @@ void GUI_Sourcer::init()
   }
   
   // enable autonotify
-//-  watcher = new KDirWatch();
   
   for(QMap<int, LVI_CDDrive*>::Iterator it = mapCDROM.begin(); it != mapCDROM.end(); ++it)
-    { /*- watcher->addDir(it.data()->getPath()); -*/ if(!it.data()->getIsSuperMounted()) dirwatcher->addDir(it.data()->getPath()); }
+    { /*- watcher->addDir(it.data()->getPath()); -*/
+       if(it.data()->getIsAutoMounted() == SUPERMOUNT)
+          supermountwatcher->addDevice(it.data()->getDevice(), it.data()->getPath());
+       else if (it.data()->getIsAutoMounted() == SUBMOUNT)
+          submountwatcher->addDevice(it.data()->getDevice(), it.data()->getPath());
+       else
+          mtabwatcher->addDir(it.data()->getPath());
+     }
   for(QMap<int, LVI_SMBShare*>::Iterator it = mapSMB.begin(); it != mapSMB.end(); ++it)
-    { /*- watcher->addDir(it.data()->getPath()); -*/ dirwatcher->addDir(it.data()->getPath()); }
+    { /*- watcher->addDir(it.data()->getPath()); -*/ mtabwatcher->addDir(it.data()->getPath()); }
   for(QMap<int, LVI_NFSExport*>::Iterator it = mapNFS.begin(); it != mapNFS.end(); ++it)
-    { /*- watcher->addDir(it.data()->getPath()); -*/ dirwatcher->addDir(it.data()->getPath()); }
+    { /*- watcher->addDir(it.data()->getPath()); -*/ mtabwatcher->addDir(it.data()->getPath()); }
   
-  dirwatcher->start();
-
   //  connect(watcher, SIGNAL(dirty(const QString&)), this, SLOT(slot_DirNotify(const QString&)));
 }
 
@@ -371,6 +412,9 @@ void GUI_Sourcer::notifyDirHasChanged(QString dir)
       mapCDROM[ID]->applyUnmount();
     } else {
       mapCDROM[ID]->applyMount(0);
+      if(mapCDROM[ID]->getIsAutoMounted() == SUBMOUNT) {
+        submountwatcher->keepMounted(mapCDROM[ID]->getDevice());
+      }
       jobman->addJob(new Job_ComputeCDROMID(mapCDROM[ID]));
     }
     return;
@@ -403,59 +447,6 @@ void GUI_Sourcer::notifyDirHasChanged(QString dir)
   }
 }
 
-// ##############################################
-// # callback for CDROM watcher
-// ##############################################
-void GUI_Sourcer::slot_DirNotify(const QString& dir)
-{
-  int ID = 0;
-  
-  for(QMap<int, LVI_CDDrive*>::Iterator it = mapCDROM.begin(); it != mapCDROM.end(); ++it)
-    if(it.data()->getPath() == dir) ID = it.key();
-  if(ID != 0) {
-    if(mapCDROM[ID]->getIsMounted()) {
-      datadispatcher->eventMediumNoMoreAvailable(mapCDROM[ID]->getCdID());
-      mapCDROM[ID]->applyUnmount();
-    } else {
-      mapCDROM[ID]->applyMount(0);
-      jobman->addJob(new Job_ComputeCDROMID(mapCDROM[ID]));
-    }
-    return;
-  }
-
-  for(QMap<int, LVI_SMBShare*>::Iterator it = mapSMB.begin(); it != mapSMB.end(); ++it)
-    if(it.data()->getPath() == dir) ID = it.key();
-  if(ID != 0) {
-//-    watcher->stopDirScan(dir);
-    //    watcher->removeDir(dir);
-    if(mapSMB[ID]->getIsMounted()) {
-      //      datadispatcher->eventMediumNoMoreAvailable(mapCDROM[ID]->getCdID());
-      mapSMB[ID]->applyUnmount();
-    } else {
-      mapSMB[ID]->applyMount();
-      //      jobman->addJob(new Job_ComputeCDROMID(mapCDROM[ID]));
-    }
-//-    watcher->startScan(true, true);
-    return;
-  }
-
-  for(QMap<int, LVI_NFSExport*>::Iterator it = mapNFS.begin(); it != mapNFS.end(); ++it)
-    if(it.data()->getPath() == dir) ID = it.key();
-  if(ID != 0) {
-//-    watcher->stopDirScan(dir);
-    //    watcher->removeDir(dir);
-    if(mapNFS[ID]->getIsMounted()) {
-      //      datadispatcher->eventMediumNoMoreAvailable(mapCDROM[ID]->getCdID());
-      mapNFS[ID]->applyUnmount();
-    } else {
-      mapNFS[ID]->applyMount();
-      //      jobman->addJob(new Job_ComputeCDROMID(mapCDROM[ID]));
-    }
-//-    watcher->startScan(true, true);
-    return;
-  }
-
-}
 
 // ##############################################
 // # check if path is mounted
@@ -487,11 +478,17 @@ bool GUI_Sourcer::isPathMounted(QString path)
   return false;
 }
 
+bool GUI_Sourcer::isPathAutoMounted(QString path)
+{
+  if(QDir(path).count() > 0) return true;
+  return false;
+}
+
 void GUI_Sourcer::notifyCDROMInserted(LVI_CDDrive *drive)
 {
   int checksum = drive->getChecksum();
   if(mapChecksum.contains(checksum)) drive->applyInfo(true, mapLabel[mapChecksum[checksum]], mapChecksum[checksum]);
-  else drive->applyInfo(false, _("<unknown>"), -1);
+  else drive->applyInfo(false, _("Unknown"), -1);
 }
 
 void GUI_Sourcer::notifySMBMounted(LVI_SMBShare *share) {
@@ -542,11 +539,11 @@ void GUI_Sourcer::notifyMediumRemoved(int id)
     mapChecksum.remove(checksum);
     mapLabel.remove(id);
     for(QMap<int, LVI_CDDrive*>::Iterator it = mapCDROM.begin(); it != mapCDROM.end(); ++it)
-      if(it.data()->getCdID() == id) it.data()->applyInfo(false, _("<unknown>"), it.data()->getChecksum());
+      if(it.data()->getCdID() == id) it.data()->applyInfo(false, _("Unknown"), it.data()->getChecksum());
   } else { // SMB
     QString device = "";
     for(QMap<QString, int>::Iterator it = mapSMBDevice.begin(); it != mapSMBDevice.end(); ++it) if(it.data() == id) device = it.key();
-    if(device != "") {
+    if(!device.isEmpty()) {
       for(QMap<int, LVI_SMBShare*>::Iterator it = mapSMB.begin(); it != mapSMB.end(); ++it) {
 	if(it.data()->getDevice() == device) it.data()->applyInfo(-1);
       }  
@@ -555,7 +552,7 @@ void GUI_Sourcer::notifyMediumRemoved(int id)
   // NFS
 //    QString device = "";
     for(QMap<QString, int>::Iterator it = mapNFSDevice.begin(); it != mapNFSDevice.end(); ++it) if(it.data() == id) device = it.key();
-    if(device != "") {
+    if(!device.isEmpty()) {
       for(QMap<int, LVI_NFSExport*>::Iterator it = mapNFS.begin(); it != mapNFS.end(); ++it) {
 	if(it.data()->getDevice() == device) it.data()->applyInfo(-1);
       }  
@@ -564,9 +561,19 @@ void GUI_Sourcer::notifyMediumRemoved(int id)
   }
 }
 
+void GUI_Sourcer::notifyMediumRenamed(MEDIUM *medium)
+{
+  mapLabel[medium->id] = medium->label;
+  for (QMap<int, LVI_CDDrive*>::Iterator it = mapCDROM.begin(); it != mapCDROM.end(); ++it) {
+    if (it.data()->getCdID() == medium->id) {
+      notifyCDROMInserted(it.data());
+    }
+  }
+}
+
 void GUI_Sourcer::notifyStartedHarddiskSynchronizing() {
   doSynchro = true;
-  lviHDD->setText(0, _("sync..."));
+  lviHDD->setText(0, QString(_("sync")) + "...");
 }
 
 void GUI_Sourcer::notifyStoppedHarddiskSynchronizing() {
@@ -598,7 +605,7 @@ void GUI_Sourcer::notifyStoppedNFSSynchronizing(LVI_NFSExport *Export) {
   Export->unlock();
 }
 
-void GUI_Sourcer::setConnectionState(bool state)
+void GUI_Sourcer::slot_setConnectionState(bool state)
 {
   isConnected = state;
   mapChecksum.clear();
@@ -612,61 +619,57 @@ void GUI_Sourcer::setConnectionState(bool state)
 }
 	
 void GUI_Sourcer::slot_menuMountCDROM() {
-	LVI_CDDrive *lvi = dynamic_cast<LVI_CDDrive*>(currentItem());
-	if(!lvi->getIsSuperMounted()) {
-	  QString cmd = "mount " + lvi->getPath() + " &";
-	  system(cmd);
-	}
-	else if(isPathMounted(lvi->getPath())) {
-	  app->lock();
-	  notifyDirHasChanged(lvi->getPath());
-	  app->unlock();
-	}
-
+    LVI_CDDrive *lvi = dynamic_cast<LVI_CDDrive*>(currentItem());
+    if(lvi->getIsAutoMounted() == NO_AUTOMOUNT) {
+        QString cmd = "mount " + lvi->getPath() + " &";
+        system(cmd.local8Bit().data());
+    } else if(lvi->getIsAutoMounted() == SUPERMOUNT) {
+        QString cmd = "ls " + lvi->getPath() + "> /dev/null &";
+        system(cmd.local8Bit().data());
+    }
 //-	KIO::mount(false, 0, lvi->getDevice(), lvi->getPath());
 }
 
 void GUI_Sourcer::slot_menuUnmountCDROM() {
-	LVI_CDDrive *lvi = dynamic_cast<LVI_CDDrive*>(currentItem());
-	if(!lvi->getIsSuperMounted()) {
-	  QString cmd = "umount " + lvi->getPath() + " &";
-	  system(cmd);
-	}
-	else if(isPathMounted(lvi->getPath())) {
-	  app->lock();
-	  notifyDirHasChanged(lvi->getPath());
-	  app->unlock();
-	  QString cmd = "eject " + lvi->getPath() + " &";
-	  system(cmd);
-	}
+    LVI_CDDrive *lvi = dynamic_cast<LVI_CDDrive*>(currentItem());
+    if(lvi->getIsAutoMounted() == NO_AUTOMOUNT) {
+        QString cmd = "umount " + lvi->getPath() + " &";
+        system(cmd.local8Bit().data());
+    } else if(lvi->getIsAutoMounted() == SUBMOUNT){
+        submountwatcher->pause();
+        QString cmd = "eject " + lvi->getPath() + " &";
+        system(cmd.local8Bit().data());
+        submountwatcher->release(lvi->getDevice());
+        submountwatcher->proceed();
+    }
 //-	KIO::unmount(lvi->getPath());
 }
 
 void GUI_Sourcer::slot_menuMountSMB() {
 	LVI_SMBShare *lvi = dynamic_cast<LVI_SMBShare*>(currentItem());
 	QString cmd = "mount " + lvi->getPath() + " &";
-	system(cmd);
+	system(cmd.local8Bit().data());
 //-	KIO::mount(false, 0, lvi->getDevice(), lvi->getPath());
 }
 
 void GUI_Sourcer::slot_menuUnmountSMB() {
 	LVI_SMBShare *lvi = dynamic_cast<LVI_SMBShare*>(currentItem());
 	QString cmd = "umount " + lvi->getPath() + " &";
-	system(cmd);
+	system(cmd.local8Bit().data());
 //-	KIO::unmount(lvi->getPath());
 }
 
 void GUI_Sourcer::slot_menuMountNFS() {
 	LVI_NFSExport *lvi = dynamic_cast<LVI_NFSExport*>(currentItem());
 	QString cmd = "mount " + lvi->getPath() + " &";
-	system(cmd);
+	system(cmd.local8Bit().data());
 //-	KIO::mount(false, 0, lvi->getDevice(), lvi->getPath());
 }
 
 void GUI_Sourcer::slot_menuUnmountNFS() {
 	LVI_NFSExport *lvi = dynamic_cast<LVI_NFSExport*>(currentItem());
 	QString cmd = "umount " + lvi->getPath() + " &";
-	system(cmd);
+	system(cmd.local8Bit().data());
 //-	KIO::unmount(lvi->getPath());
 }
 
