@@ -20,11 +20,11 @@
 
 #include "prokyon.h"
 
-#include "jobbase.h"
+#include "jobbasetagmanipulation.h"
 
 #include "lvi_cddrive.h"
 
-class Job_AppendCDROMToDatabase : public JobBase  {
+class Job_AppendCDROMToDatabase : public JobBaseTagManipulation  {
 public: 
 	Job_AppendCDROMToDatabase(LVI_CDDrive *_drive);
 	
