@@ -20,11 +20,11 @@
 
 #include "prokyon.h"
 
-#include "jobbase.h"
+#include "jobbasetagmanipulation.h"
 
 #include "lvi_nfsexport.h"
 
-class Job_AppendNFSToDatabase : public JobBase  {
+class Job_AppendNFSToDatabase : public JobBaseTagManipulation  {
  public: 
   Job_AppendNFSToDatabase(LVI_NFSExport *_Export);
 
