@@ -20,11 +20,11 @@
 
 #include "prokyon.h"
 
-#include "jobbase.h"
+#include "jobbasetagmanipulation.h"
 
 #include "lvi_smbshare.h"
 
-class Job_AppendSMBToDatabase : public JobBase  {
+class Job_AppendSMBToDatabase : public JobBaseTagManipulation  {
  public: 
   Job_AppendSMBToDatabase(LVI_SMBShare *_share);
 
