@@ -6,7 +6,7 @@
 
 class Job_GeneratePlaylist : public JobBase  {
 public:
-	Job_GeneratePlaylist(int _numberTracks, int _minTrackLength, int _maxTrackLength, bool _onlyFavouriteArtists);
+	Job_GeneratePlaylist(int _numberTracks, int _minTrackLength, int _maxTrackLength, bool _onlyFavouriteArtists, bool _onlyHarddisk, int _minRating);
 
     int type() { return JOB_GENERATE_PLAYLIST; }
 
@@ -19,6 +19,8 @@ private:
 	int minTrackLength;
 	int maxTrackLength;
     bool onlyFavouriteArtists;
+    bool onlyHarddisk;
+    int minRating;
 };
 
 #endif
