@@ -64,11 +64,15 @@ DOUBLE CHANGED = Files changed both internally and externally
 
 #include <qptrlist.h>
 #include <qstringlist.h>
-#include <qqueue.h>
+#include <qptrqueue.h>
 #include <qdir.h>
 
 #include <iostream>
 
+#ifdef EMBEDDED 
+  #include <mysql.h>
+#endif
+
 using namespace std;
 
 Job_SynchronizeWithDatabase::Job_SynchronizeWithDatabase(int _modus, int _parameter)
@@ -98,6 +102,21 @@ void Job_SynchronizeWithDatabase::run()
 {
   int error;
 
+  if (verbose==8) qWarning( "    >run() tread:%p type:%s HANDLE:%p", this, JOB_CODE[type()], currentThread() );
+
+#ifdef EMBEDDED 
+  mysql_thread_init(); 
+#endif
+
+  // save listview's selection  for later restore 
+  QString artistBuf = QString::null;
+
+  if ( QListViewItem* lvi = gui->getSelector()->getTabArtist()->selectedItem() )
+    if ( dynamic_cast<QListViewItem*>( lvi->parent() ) ) 
+      artistBuf = lvi->parent()->text(0); 
+    else 
+      artistBuf = lvi->text(0); 
+ 
   app->lock();
   datadispatcher->eventAddEditLock();
   switch(mediumType) {
@@ -108,7 +127,7 @@ void Job_SynchronizeWithDatabase::run()
   }
   app->unlock();
   
-  QQueue<QDir>	directoryQueue;
+  QPtrQueue<QDir>	directoryQueue;
   
   QStringList userDirectories;
 
@@ -131,6 +150,11 @@ void Job_SynchronizeWithDatabase::run()
   case MEDIUM_NFS:      directoryQueue.enqueue(new QDir(Export->getPath())); break;
   }
 
+  QString previousStatusMode = gui->getStatusInfo( 5 );
+  qApp->lock();
+  gui->setStatusInfo( " Mode: Sync.... ", 5);
+  qApp->unlock();
+
   // get all files from medium
   while(!directoryQueue.isEmpty()) {
     QDir *curr = directoryQueue.dequeue();
@@ -138,6 +162,7 @@ void Job_SynchronizeWithDatabase::run()
     delete curr;
   }
 
+
   // Remove duplicates entry if same files are found twice in path 
   QMap<QString,TRACK*> DBByPath;
   QPtrListIterator<TRACK> it( *localFiles );
@@ -166,7 +191,7 @@ void Job_SynchronizeWithDatabase::run()
   storedFiles = database->getTracksByMedium(getMediumID());
   error = database->getError();
   database->unlock();
-  
+
   // *********************************************
   // *** SYNCHRONISATION harddisk <-> database ***
   // *********************************************
@@ -271,8 +296,8 @@ void Job_SynchronizeWithDatabase::run()
     QMap<QString,TRACK*> DBByModified;
     QMap<QString,int> droplist;
     for(TRACK *curr = storedFiles->first(); curr != 0; curr = storedFiles->next()) {
-      QString key;
-      key.sprintf("%s%d", curr->lastModified.toString().latin1(), curr->size);
+      QString key( "%1%2" );
+      key.arg( curr->lastModified.toString().local8Bit().data() ).arg( curr->size );
       if(DBByModified.contains(key)) {
 	droplist[key] = 0;
 	DBByModified.remove(key);
@@ -283,8 +308,8 @@ void Job_SynchronizeWithDatabase::run()
     
     QMap<QString,TRACK*> LocalByModified;
     for(TRACK *curr = localFiles->first(); curr != 0; curr = localFiles->next()) {
-      QString key;
-      key.sprintf("%s%d", curr->lastModified.toString().latin1(), curr->size);
+      QString key( "%1%2" );
+      key.arg( curr->lastModified.toString().local8Bit().data() ).arg( curr->size );
       if(LocalByModified.contains(key)) {
 	droplist[key] = 0;
 	LocalByModified.remove(key);
@@ -448,10 +473,9 @@ void Job_SynchronizeWithDatabase::run()
 
     // Quick fix when directory is changed in configuration.cpp
     gui->getSelector()->clear();
-    datadispatcher->eventNewArtistAlbumBasis(database->getArtistAlbumBasis());
+    datadispatcher->eventNewArtistAlbumBasis(database->getArtistAlbumBasis(), false);
     datadispatcher->eventNewFavouritesBasis(database->getFavouritesBasis());
-    datadispatcher->eventNewSamplerBasis(database->getSamplerBasis());
-    datadispatcher->eventNewLocalDeltaBasis(database->getLocalDeltaBasis());
+    datadispatcher->eventNewAlbumBasis(database->getAlbumBasis());
     datadispatcher->eventNewLocalAlbumDeltaBasis(database->getLocalAlbumDeltaBasis());
     datadispatcher->eventNewMediumBasis(database->getMediumBasis());
     datadispatcher->eventNewPlaylistBasis(database->getPlaylistBasis());
@@ -511,9 +535,31 @@ void Job_SynchronizeWithDatabase::run()
   delete internChangedFiles;
   if(storedFiles != 0) delete storedFiles;
 
+  qApp->lock();
+
+  // restore listview selection 
+  LVI_Album* lvi1 = 0;
+  QListView* listArtist = gui->getSelector()->getTabArtist();
+  if (!artistBuf.isNull() ) { 
+    lvi1 = dynamic_cast<LVI_Album*>( listArtist->findItem ( artistBuf, 0) ); 
+    if (lvi1) {
+      listArtist->setCurrentItem( lvi1 );
+      listArtist->setSelected( lvi1, true );
+    }
+  }
+
+  gui->setStatusInfo( previousStatusMode, 5 );
+  qApp->unlock();
+
   jobman->lock();
   jobman->jobDone(this);
   jobman->unlock();
+#ifdef EMBEDDED 
+  mysql_thread_end(); 
+#endif
+
+  if (verbose==8) qWarning( "    >exit() thread:%p type:%s HANDLE: %p", this, JOB_CODE[type()], currentThread() );
+
   exit();
 }
 
