@@ -18,26 +18,16 @@
 #ifndef JOBBASE_H
 #define JOBBASE_H
 
-#include "prokyon.h"
-
 #include <qthread.h>
-#include <qdir.h>
 
 class JobBase: public QThread  {
 public:
 	JobBase() {}
 	~JobBase() {}	
-	
 	virtual int type() = 0;
 
 protected:
 	virtual void run() = 0;
-
-	QList<TRACK> *getFileList(QDir *directory, bool append, QList<TRACK> *files=0);
-	QList<TRACK> *readTags(QList<TRACK> *files, bool lazyRead=false);
-	QList<TRACK> *readTagsOnly(QList<TRACK> *files);
-	QList<TRACK> *writeTags(QList<TRACK> *files);
-
 };
 
 #endif
