@@ -33,13 +33,4 @@ public:
 	~Tagger_OggVorbis();
 };
 
-class VorbisComment {
- public: 
-  VorbisComment(vorbis_comment *vc );
-  QString  getvorbiscomment(char *tag);
-  ~VorbisComment();
- private:
-  QMap<QString, QString> comments;
-};
-
 #endif
